<?php

declare(strict_types=1);

namespace Reloadify\Controller;

use Reloadify\Service\AbandonedCartService;
use Shopware\Core\Checkout\Cart\AbstractCartPersister;
use Shopware\Core\Checkout\Cart\Cart;
use Shopware\Core\Checkout\Cart\SalesChannel\CartService;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Controller\StorefrontController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\Cache;
use Symfony\Component\Routing\Attribute\Route;

#[Route(defaults: ['_routeScope' => ['storefront']])]
class AbandonedCartController extends StorefrontController
{
    protected AbandonedCartService $abandonedCartService;
    protected CartService $cartService;
    protected AbstractCartPersister $cartPersister;

    public function __construct(
        AbandonedCartService   $abandonedCartService,
        CartService            $cartService,
        AbstractCartPersister $cartPersister
    )
    {
        $this->abandonedCartService = $abandonedCartService;
        $this->cartService = $cartService;
        $this->cartPersister = $cartPersister;
    }

    /**
     * Claim a cart by its id in reloadify_abandoned_cart e.g. '635948eb99154696a9a8a0532a6096bb'
     */
    #[Route(path: '/claim-cart/{id}', name: 'reloadify.frontend.claim.cart', methods: ['GET'])]
    #[Cache(mustRevalidate: true)]
    public function claimCart(SalesChannelContext $context, string $id): Response
    {
        //Get abandoned cart from database with id
        $abandonedCartEntity = $this->abandonedCartService->getAbandonedCartWithId($id, $context->getContext());

        if (empty($abandonedCartEntity)) {
            $this->addFlash(self::WARNING, $this->trans('reloadify.checkout.noCartFoundMessage'));
            return $this->forwardToRoute('frontend.home.page', ['success' => false]);
        }

        /** @var Cart $cart */
        $cart = unserialize($abandonedCartEntity->getCart());

        $cart->setToken($context->getToken());
        $this->abandonedCartService->setAbandonedCartToken($id,$context->getToken(),$context->getContext());
        $this->cartService->setCart($cart);
        $this->cartService->recalculate($cart, $context);
        $this->addFlash(self::INFO, $this->trans('reloadify.checkout.cartAppliedMessage'));

        return $this->forwardToRoute('frontend.checkout.cart.page');
    }
}
