<?php

declare(strict_types=1);

namespace Reloadify\Entity\AbandonedCart;

use Shopware\Core\Checkout\Customer\CustomerDefinition;
use Shopware\Core\Framework\DataAbstractionLayer\EntityDefinition;
use Shopware\Core\Framework\DataAbstractionLayer\Field\FkField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\ApiAware;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\PrimaryKey;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\Required;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\SearchRanking;
use Shopware\Core\Framework\DataAbstractionLayer\Field\Flag\WriteProtected;
use Shopware\Core\Framework\DataAbstractionLayer\Field\FloatField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\IdField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\JsonField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\LongTextField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\ManyToOneAssociationField;
use Shopware\Core\Framework\DataAbstractionLayer\Field\StringField;
use Shopware\Core\Framework\DataAbstractionLayer\FieldCollection;
use Shopware\Core\System\Currency\CurrencyDefinition;
use Shopware\Core\System\Language\LanguageDefinition;
use Shopware\Core\System\SalesChannel\SalesChannelDefinition;

class AbandonedCartDefinition extends EntityDefinition
{

    public const ENTITY_NAME = 'reloadify_abandoned_cart';

    public const TOKEN_FIELD = 'token';
    public const CART_FIELD = 'cart';
    public const CUSTOMER_ID_FIELD = 'customerId';

    public function getEntityName(): string
    {
        return self::ENTITY_NAME;
    }

    protected function defineFields(): FieldCollection
    {
        return new FieldCollection([
            (new IdField('id','id'))->addFlags(new Required(),new PrimaryKey(), new ApiAware()),
            (new StringField('token',self::TOKEN_FIELD))->addFlags(new Required()),
            (new LongTextField('cart',self::CART_FIELD))->addFlags(new Required())->removeFlag(ApiAware::class),
            (new JsonField('payload','payload'))->addFlags(new Required(), new ApiAware()),
            //Fk Fields
            (new FkField('customer_id','customerId',CustomerDefinition::class))
                ->addFlags(new Required(),new ApiAware()),
            (new FkField('sales_channel_id','salesChannelId',SalesChannelDefinition::class))
                ->addFlags(new Required(),new ApiAware()),
            (new FkField('language_id', 'languageId', LanguageDefinition::class))
                ->addFlags(new ApiAware(), new Required()),
            (new FkField('currency_id', 'currencyId', CurrencyDefinition::class))
                ->addFlags(new ApiAware(), new Required()),
            (new FloatField('amount_total', 'amountTotal'))
                ->addFlags(new ApiAware(), new WriteProtected(), new SearchRanking(SearchRanking::MIDDLE_SEARCH_RANKING)),
            (new FloatField('amount_net', 'amountNet'))
                ->addFlags(new ApiAware(), new WriteProtected()),
            //Associations
            (new ManyToOneAssociationField('customer','customer_id',CustomerDefinition::class)),
            (new ManyToOneAssociationField('salesChannel','sales_channel_id',SalesChannelDefinition::class)),
            (new ManyToOneAssociationField('language', 'language_id', LanguageDefinition::class, 'id', false))->addFlags(new ApiAware()),
            (new ManyToOneAssociationField('currency', 'currency_id', CurrencyDefinition::class, 'id', false))->addFlags(new ApiAware()),
        ]);
    }

    public function getEntityClass(): string
    {
        return AbandonedCartEntity::class;
    }

    public function getCollectionClass(): string
    {
        return AbandonedCartCollection::class;
    }
}
