<?php

namespace Reloadify\Subscriber;

use Doctrine\DBAL\ConnectionException;
use Reloadify\Event\AbandonedCartDeleteEvent;
use Reloadify\Service\AbandonedCartService;
use Shopware\Core\Checkout\Cart\Event\CartSavedEvent;
use Shopware\Core\Checkout\Cart\Event\CheckoutOrderPlacedEvent;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Core\Content\Flow\Exception\CustomerDeletedException;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class CartSubscriber implements EventSubscriberInterface
{
    protected AbandonedCartService $abandonedCartService;

    public function __construct(AbandonedCartService $abandonedCartService)
    {
        $this->abandonedCartService = $abandonedCartService;
    }

    /**
     * @inheritDoc
     */
    public static function getSubscribedEvents()
    {
        return [
            CartSavedEvent::class           => 'onSaveCart',
            AbandonedCartDeleteEvent::class => 'onCartDeleted',
            CheckoutOrderPlacedEvent::class => 'onCheckoutOrderPlaced',
        ];
    }

    /**
     * @param AbandonedCartDeleteEvent $event
     * @return void
     */
    public function onCartDeleted(AbandonedCartDeleteEvent $event)
    {
        $this->abandonedCartService->deleteAbandonedCart(
            $event->getToken(),
            $event->getContext()->getContext()
        );
    }

    /**
     * @param CheckoutOrderPlacedEvent $event
     * @return void
     */
    public function onCheckoutOrderPlaced(CheckoutOrderPlacedEvent $event)
    {
        try {
            $this->abandonedCartService->deleteAbandonedCartByCustomerId(
                $event->getCustomerId(),
                $event->getContext()
            );
        }
        catch (CustomerDeletedException $e) {
            //DO nothing
        }
    }

    /**
     * @param CartSavedEvent $event
     * @return void
     * @throws ConnectionException
     */
    public function onSaveCart(CartSavedEvent $event)
    {
        $cart = $event->getCart();
        $salesChannelContext = $event->getSalesChannelContext();
        $customer = $salesChannelContext->getCustomer();

        if (!$customer instanceof CustomerEntity) {
            return;
        }

        $this->abandonedCartService->saveAbandonedCart(
            $cart,
            $cart->getToken(),
            $customer->getId(),
            $salesChannelContext->getSalesChannelId(),
            $salesChannelContext->getLanguageId(),
            $salesChannelContext->getCurrencyId(),
            $cart->getPrice()->getTotalPrice(),
            $cart->getPrice()->getNetPrice(),
            $event->getContext()
        );
    }
}
